/**
 * EasyBeans
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestNGInject.java 6135 2012-01-24 17:02:48Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.testng4osgi.inject;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.inject.Inject;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Manages dependency injection in the TestNG test classes. 
 * @author Florent Benoit
 */
public class TestNGInject {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(TestNGInject.class);

    /**
     * Bundle Context.
     */
    private BundleContext bundleContext;

    /**
     * Build a new injector.
     * @param bundleContext the given bundle context
     */
    public TestNGInject(BundleContext bundleContext) {
        this.bundleContext = bundleContext;
    }

    /**
     * Search a value for the given class
     * @param clazz the clazz of the field/setter method
     * @return the value found
     */
    protected Object getValue(String name, Class<?> analyzedClass, Class<?> searchedClass) {
        if (BundleContext.class.equals(searchedClass)) {
            return bundleContext;
        }
        
        // Now search the service in the bundle context
        ServiceReference<?> serviceReference= bundleContext.getServiceReference(searchedClass);
        if (serviceReference != null) {
            Object service = bundleContext.getService(serviceReference);
            if (service != null) {
                return service;
            }
        }

        throw new IllegalStateException("Unable to find value for the '" + name + "' field/setter with the searched class '"
                + searchedClass + "' in the analyzed class '" + analyzedClass + "'.");
    }

    
    /**
     * Performs dependency injection in the given instance
     * @param instance the instance to manage
     * @return updated instance
     */
    public Object inject(Object instance) {
        if (instance == null) {
            return instance;
        }
        
        // Search on fields
        Field[] fields = instance.getClass().getDeclaredFields();
        if (fields != null) {
            for (Field field : fields) {
                Inject inject = field.getAnnotation(Inject.class);
                if (inject != null) {
                    boolean isAccessible = field.isAccessible();
                    try {
                        field.setAccessible(true);
                        Object value = getValue(field.getName(), instance.getClass(), field.getType());
                        LOGGER.debug("Injecting value ''{0}'' in field ''{1}''" , value, field);
                        if (value != null) {
                            field.set(instance, value);
                        }
                    } catch (IllegalArgumentException e) {
                        throw new IllegalStateException("Cannot inject field", e);
                    } catch (IllegalAccessException e) {
                        throw new IllegalStateException("Cannot inject field", e);
                    } finally {
                        field.setAccessible(isAccessible);
                    }
                }
            }
        }
        
        // Search on methods
        Method[] methods = instance.getClass().getDeclaredMethods();

        if (methods != null) {
            for (Method method : methods) {
                // Valid method ?
                Class<?>[] parameterTypes = method.getParameterTypes();
                if (parameterTypes == null || parameterTypes.length != 1) {
                    continue;
                }
                
                // Get type
                Class<?> parameterType = parameterTypes[0];
                
                Inject inject = method.getAnnotation(Inject.class);
                if (inject != null) {
                    boolean isAccessible = method.isAccessible();
                    
                    
                    try {
                        method.setAccessible(true);
                        Object value = getValue(method.getName(), instance.getClass(), parameterType);
                        LOGGER.debug("Injecting value ''{0}'' in field ''{1}''" , value, method);
                        if (value != null) {
                            method.invoke(instance, value);
                        }
                    } catch (IllegalArgumentException e) {
                        throw new IllegalStateException("Cannot inject method", e);
                    } catch (IllegalAccessException e) {
                        throw new IllegalStateException("Cannot inject method", e);
                    } catch (InvocationTargetException e) {
                        throw new IllegalStateException("Cannot inject method", e);
                    } finally {
                        method.setAccessible(isAccessible);
                    }
                }
            }
        }
        return instance;
    }

}
