/**
 * EasyBeans
 * Copyright (C) 2012 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestNGLauncher.java 6135 2012-01-24 17:02:48Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.testng4osgi.launcher;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.maven.osgi.launcher.test.api.ITestConfiguration;
import org.ow2.util.maven.osgi.launcher.test.api.ITestLauncher;
import org.ow2.util.maven.osgi.launcher.test.api.ITestReport;
import org.ow2.util.testng4osgi.factory.TestNGObjectFactory;
import org.ow2.util.testng4osgi.listener.TestNGTestListener;
import org.testng.TestNG;

/**
 * Allows to start the TestNG tests. 
 * @author Florent Benoit
 */
public class TestNGLauncher implements ITestLauncher {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(TestNGLauncher.class);

    
    /**
     * List of TestNG objects.
     */
    private Map<BundleContext, EnhancedTestNG> testNGs = null;
    
    /**
     * Report listener.
     */
    private TestNGTestListener reportListener = null;
    
    /**
     * Default constructor.
     */
    public TestNGLauncher() {
        this.testNGs = new HashMap<BundleContext, EnhancedTestNG>();
        this.reportListener  = new TestNGTestListener();
    }
    
    /**
     * Adding new tests for the given bundle context.
     * @param bundleContext the bundle context of the given classes
     * @param classes the set of classes to add
     */
    public void addTests(BundleContext bundleContext, List<Class<?>> classes) {
        
        // new TestNG engine for each set of classes
        EnhancedTestNG testNG = new EnhancedTestNG(bundleContext);
        
        // define classes
        testNG.setOSGiClasses(classes);
        
        // Set our own factory in order to be able to manage dependency injection
        testNG.setObjectFactory(new TestNGObjectFactory(bundleContext));
     
        // Listener used to manage failures
        testNG.addListener(reportListener);
        
        //TODO: remove debug
        // Increase verbosity for now
        //testNG.setVerbose(10);
        
        // Add engine to the set
        testNGs.put(bundleContext, testNG);
        
    }
    
    /**
     * Run the given tests by using the given configuration.
     * @param configuration the test configuration
     * @return a report
     */
    public ITestReport runTests(ITestConfiguration configuration) {
        
        // Update configuration
        for (TestNG testNG : testNGs.values()) {
            if (configuration.getOutputDirectory() != null) {
                testNG.setOutputDirectory(configuration.getOutputDirectory().getAbsolutePath());
            }
        }

        // Run
        for (TestNG testNG : testNGs.values()) {
            testNG.run();
        }
        
        // Analyze results
        int status = 0;
        for (TestNG testNG : testNGs.values()) {
            status = status | testNG.getStatus();
        }
        
        LOGGER.info("Report : Run ''{0}'', Failures : ''{1}'', Skipped : ''{2}'', Success : ''{3}''",
                reportListener.getTestStart(), reportListener.getFailures() ,reportListener.getSkipped() ,
                reportListener.getTestStart() ,reportListener.getTestSuccess());
        
        
        return new TestReport(0 == status);
    }

}
